/* 
 *
 * Copyright (c) <2010> Comarch S.A. and/or its subsidiary(-ies).
 * All rights reserved.
 * This component and the accompanying materials are made available
 * under the terms of the License "Eclipse Public License v1.0"
 * which accompanies this distribution, and is available
 * at the URL "http://www.eclipse.org/legal/epl-v10.html".
 *
 * Original Contributors:
 * Comarch S.A. - original contribution.
 *
 * Contributors:
 *
 * Description:
 *
 */

#ifndef HFPINCOMINGCALLACCEPTANCE_H
#define HFPINCOMINGCALLACCEPTANCE_H

#include <e32base.h>

#include "hfpfeaturecommons.h"

class CHsHFPCommand;
class CHsHFPSettings;

/**
 * @brief Represents "Accept an incoming voice call" feature
 */
class CHsHFPIncomingCallAcceptance : public CBase,
        public MHsHFPProcedureCommons
    {
public:
    /**
     * Two-phased constructor.
     * 
     * @param aObserver feature observer
     * 
     * @return instance of class
     */
    static CHsHFPIncomingCallAcceptance* NewL(
            MHsHFPFeatureProviderObserver* aObserver);

    /**
     * Two-phased constructor.
     * 
     * @param aObserver feature observer
     * 
     * @return instance of class
     */
    static CHsHFPIncomingCallAcceptance* NewLC(
            MHsHFPFeatureProviderObserver* aObserver);

    /**
     * Destructor.
     */
    ~CHsHFPIncomingCallAcceptance();

public:
    //Method inherited from MHsHFPProcedureCommons
    TInt ProcessCommand(const CHsHFPCommand &aInputCmd,
            CHsHFPCommand &aOutputCmd);

private:

    /**
     * Constructor for performing 1st stage construction
     * 
     * @param aObserver feature observer
     */
    CHsHFPIncomingCallAcceptance(MHsHFPFeatureProviderObserver* aObserver);

    /**
     * Constructor for performing 2nd stage construction
     */
    void ConstructL();

    /**
     * Checks if received CIEV is proper
     * 
     * @param aCommand AT command
     * @param aIndicatorDes expected indicator kind
     * @param aIndicatorValue expected indicator value
     */
    TBool IsProperCIEV(const CHsHFPCommand* aCommand,
            const TDesC8& aIndicatorDes, const TInt aIndicatorValue);

private:
    /** Pointer to observer */
    MHsHFPFeatureProviderObserver* iObserver;

    /** Denotes if OK command is expected */
    TBool iWaitingForOK;

    /** Denotes if ATA command is expected */
    TBool iWaitingForATA;

    /** Denotes if CIEV with call indicator command is expected */
    TBool iWaitingForCIEVCall1;

    /** Denotes if CIEV with callsetup indicator command is expected */
    TBool iWaitingForCIEVCallsetup0;

    /** Profile setting */
    CHsHFPSettings* iSettings;
    };

#endif // HFPINCOMINGCALLACCEPTANCE_H
